#ifndef __AUTOMATON_H
#define __AUTOMATON_H

#include "Regexp.h"

#include <iostream>
#include <stdlib.h>

#include <vector>
#include <map>
#include <set>

using namespace std;

class State
{
 public:
  State() {}
};

class Label
{
 public:
  int is_epsilon;
  int is_letter;
  char letter;

  Label() { is_epsilon = true; is_letter = false; }
  Label(char l) { is_epsilon = false; is_letter = true; letter = l; }
};

class Automaton
{
 public:
  set<char> alphabet;

  vector<State*> state;
  map<State*, map<Label*, State*> > succ;

  State* initial_state;
  vector<State*> final_state;

  State* default_state; //"puits", after completion only.

  Automaton(set<char> sigma) { alphabet = sigma; }
  Automaton(set<char> sigma, Regexp* r); //Thompson construction

  set<State*> epsilon_closure(State* s);
  set<State*> epsilon_closure(set<State*> v);

  Automaton* determinize();

  void complete();

  Automaton* minimize();

  int state_rank(State* s);

  pair<pair< vector<vector<int> > , map<char,int> > , vector<int> >
    transition_table(set<char> markers);
  //==> ((transition table, mapping letter -> rank) , mapping state -> token)

  int is_final(State* s);

  void print_code(ostream& sout);

  void print_dot(ostream& sout);

 private:
  set<State*> epsilon_closure_(State* s, set<State*> reached);
};

ostream& operator <<(ostream& sout, Label& label);

#endif
